#region Copyright notice and license

// Copyright 2019 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#endregion

using Grpc.Core;
using Grpc.Health.V1;
using Grpc.Net.Client;

using var channel = GrpcChannel.ForAddress("https://localhost:5001");
var client = new Health.HealthClient(channel);

Console.WriteLine("Watching health status");
Console.WriteLine("Press any key to exit...");

var cts = new CancellationTokenSource();
using var call = client.Watch(new HealthCheckRequest { Service = "" }, cancellationToken: cts.Token);
var watchTask = Task.Run(async () =>
{
    try
    {
        await foreach (var message in call.ResponseStream.ReadAllAsync())
        {
            Console.WriteLine($"{DateTime.Now}: Service is {message.Status}");
        }
    }
    catch (RpcException ex) when (ex.StatusCode == StatusCode.Cancelled)
    {
        // Handle cancellation exception.
    }
});

Console.ReadKey();
Console.WriteLine("Finished");

cts.Cancel();
await watchTask;
